#
# Examples for PyOpenGL (and PIL)
#
# Author: Alex V. Boreskoff
#

from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
#from OpenGL.GL.EXT.fog_coord import *
#from OpenGL.GL.ARB.vertex_shader import *
#from OpenGL.GL.ARB.fragment_shader import *
from OpenGL.GL.ARB.texture_cube_map import *
import Image 				# PIL
import sys

angle    = 0.0
angle2   = 0.0
texture  = 0
object   = 0
reflMode = GL_REFLECTION_MAP_ARB

def loadTexture ( fileName ):
	image  = Image.open ( fileName )
	width  = image.size [0]
	height = image.size [1]
	image  = image.tostring ( "raw", "RGBX", 0, -1 )
	
	texture = glGenTextures ( 1 )
	glBindTexture     ( GL_TEXTURE_2D, texture )   # 2d texture (x and y size)
	glPixelStorei     ( GL_UNPACK_ALIGNMENT,1 )
	glTexParameterf   ( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT )
	glTexParameterf   ( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT )
	glTexParameteri   ( GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR )
	glTexParameteri   ( GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR )
	gluBuild2DMipmaps ( GL_TEXTURE_2D, 3, width, height, GL_RGBA, GL_UNSIGNED_BYTE, image )
	
	return texture

def loadCubemap ( faces, path = "" ):
	texture = glGenTextures ( 1 )
	target  = GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB
	glBindTexture     ( GL_TEXTURE_CUBE_MAP_ARB, texture )   # 2d texture (x and y size)
	glPixelStorei     ( GL_UNPACK_ALIGNMENT,1 )
	glTexParameteri   ( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,     GL_REPEAT )
	glTexParameteri   ( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,     GL_REPEAT )
	glTexParameteri   ( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR )
	glTexParameteri   ( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR )
	glEnable          ( GL_TEXTURE_CUBE_MAP_ARB )
	
	for f in faces:
		if path != "":
			file = path + "/" + f
		else:
			file = f
		
		image  = Image.open ( file )
		width  = image.size [0]
		height = image.size [1]
		image  = image.tostring ( "raw", "RGBX", 0, -1 )
		gluBuild2DMipmaps ( target, 3, width, height, GL_RGBA, GL_UNSIGNED_BYTE, image )
		target = target + 1
	
	return texture

def extensionInit ():
	""" Determine if the fog coord extentsion is availble """

	# After calling this, we will be able to invoke glFogCoordEXT ()
	if not glInitTextureCubeMapARB ():
		print "ARB_texture_cubemap not supported !"
		sys.exit ( 1 )

def init ():
	glClearColor ( 0.0, 0.0, 0.0, 0.0 )
	glClearDepth ( 1.0 )				
	glDepthFunc  ( GL_LEQUAL )
	glEnable     ( GL_DEPTH_TEST )
	glHint       ( GL_POLYGON_SMOOTH_HINT,         GL_NICEST )
	glHint       ( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST )

def reshape ( width, height ):
	glViewport     ( 0, 0, width, height )
	glMatrixMode   ( GL_PROJECTION )
	glLoadIdentity ()
	gluPerspective ( 60.0, float(width)/float (height), 1.0, 60.0 )
	glMatrixMode   ( GL_MODELVIEW )
	glLoadIdentity ()
	gluLookAt      ( 0.0, 0.0, 0.0, 5.0, 5.0, 5.0, 0.0, 1.0, 0.0 )

def display ():
	global texture, reflMode
	
	glClear   ( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT )
	glEnable  ( GL_TEXTURE_CUBE_MAP_ARB )
	glEnable  ( GL_TEXTURE_GEN_S )
	glEnable  ( GL_TEXTURE_GEN_T )
	glEnable  ( GL_TEXTURE_GEN_R )
	glTexGeni ( GL_S, GL_TEXTURE_GEN_MODE, reflMode )
	glTexGeni ( GL_T, GL_TEXTURE_GEN_MODE, reflMode )
	glTexGeni ( GL_R, GL_TEXTURE_GEN_MODE, reflMode )
	
	glBindTexture   ( GL_TEXTURE_CUBE_MAP_ARB, texture )
	glPushMatrix    ()
	glTranslatef    ( 2, 2, 2 )
	glRotatef       ( angle,  1, 1, 0 )
	glRotatef       ( angle2, 0, 1, 1 )
	
	if object == 0:
		glutSolidTeapot ( 1 )
	elif object == 1:
		glutSolidSphere ( 1, 30, 30 )
	elif object == 2:
		glutSolidCube ( 1 )
	elif object == 3:
		glutSolidCone ( 0.7, 1.2, 30, 30 )
	elif object == 4:
		glutSolidTorus ( 0.5, 1, 40, 40 )
	elif object == 5:
		glutSolidDodecahedron ()
	elif object == 6:
		glutSolidOctahedron ()
	elif object == 7:
		glutSolidTetrahedron ()
	elif object == 8:
		glutSolidIcosahedron ();
	
	glPopMatrix     ()
	glutSwapBuffers ()

def keyPressed ( *args ):
	global object, reflMode
	
	key = args [0]
	if key == '\033':
		sys.exit ()
	elif key == 'n' or key == 'N':
		reflMode = GL_NORMAL_MAP_ARB
	elif key == 'r' or key == 'R':
		reflMode = GL_REFLECTION_MAP_ARB
	elif key == '0':
		object = 0
	elif key == '1':
		object = 1
	elif key == '2':
		object = 2
	elif key == '3':
		object = 3
	elif key == '4':
		object = 4
	elif key == '5':
		object = 5
	elif key == '6':
		object = 6
	elif key == '7':
		object = 7
	elif key == '8':
		object = 8
	elif key == '+':
		object = object + 1
		if object > 8:
			object = 0
	elif key == '-':
		object = object-1
		if object < 0:
			object = 8

def animate ():
	global angle, angle2
	
	angle  = 0.04 * glutGet ( GLUT_ELAPSED_TIME )
	angle2 = 0.01 * glutGet ( GLUT_ELAPSED_TIME )
	
	glutPostRedisplay ()

def main ():
	global texture
	
	glutInit               ( sys.argv )
	glutInitDisplayMode    ( GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH )
	glutInitWindowSize     ( 500, 500 )
	glutInitWindowPosition ( 0, 0 )
	
	glutCreateWindow ( "ARB_texture_cubemap demo" )
	glutDisplayFunc  ( display )
	glutIdleFunc     ( animate )
	glutReshapeFunc  ( reshape )
	glutKeyboardFunc ( keyPressed )

	init          ()
	extensionInit ();
	
	texture = loadCubemap ( ( "cm_left.tga", "cm_right.tga", "cm_top.tga", "cm_bottom.tga", "cm_back.tga", "cm_front.tga" ), "../../Textures/Cubemaps" )
	print texture
	glutMainLoop()

print "Hit ESC key to quit."
main()
    	
