#
# Examples for PyOpenGL (and PIL) and GLSL 
# Python port of glass-2 example from "Shaders" book
#
# Author: Alex V. Boreskoff
#

from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
from OpenGL.GL.ARB.texture_cube_map import *
import Image 				# PIL
import sys
import math
import GlslProgram
from utils import *

decalMap      = None
stoneMap      = None
teapotMap     = None
fringeMap     = None
reflectionMap = None
angle         = 0.0
rot           = 0.0
rot2          = 0.0
lastTime      = 0.0
useFilter     = 1
useRotation   = 1
object        = 0
eta           = ( 1.05, 1.1, 1.15 )
program       = None

def init ():
	glClearColor ( 0.0, 0.0, 0.0, 0.0 )
	glClearDepth ( 1.0 )				
	glDepthFunc  ( GL_LEQUAL )
	glEnable     ( GL_DEPTH_TEST )
	glEnable     ( GL_TEXTURE_2D )
	glHint       ( GL_POLYGON_SMOOTH_HINT,         GL_NICEST )
	glHint       ( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST )

def reshape ( width, height ):
	glViewport     ( 0, 0, width, height )
	glMatrixMode   ( GL_PROJECTION )
	glLoadIdentity ()
	gluPerspective ( 60.0, float(width)/float (height), 1.0, 60.0 )
	glMatrixMode   ( GL_MODELVIEW )
	glLoadIdentity ()
	gluLookAt      ( 0.0, 0.0, 0.0, 0.0, 2.0, 0.0, 0.0, 0.0, 1.0 )

def display ():
	global decalMap, stoneMap, teapotMap, fringeMap, reflectionMap, program
	
	glClear        ( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT )
	glMatrixMode   ( GL_MODELVIEW )
	glPushMatrix   ()
	
	glRotatef   ( rot,  0, 0, 1 )
	glRotatef   ( rot2, 1, 0, 0 )
	
	mv = glGetFloatv ( GL_MODELVIEW_MATRIX )
	t  = [ mv [i][j] for i in range ( 4 ) for j in range ( 4 )]	# reorder mv into single list
	
	drawCubicMappedBox ( ( 4, 4, 4 ), reflectionMap )
	
	program.bind ()
	program.setUniformMatrix ( "cubeMapMatrix", t )
	
	glPopMatrix  ()
	glPushMatrix ()
	glTranslatef ( 0, 2, 0 );
	glRotatef    ( angle * 45.3, 1, 0, 0 );
	glRotatef    ( angle * 57.2, 0, 1, 0 );
	
	if object == 0:
		glutSolidTeapot ( 0.6 )
	elif object == 1:
		glutSolidSphere ( 0.6, 30, 30 )
	elif object == 2:
		glutSolidCube ( 0.8 )
	elif object == 3:
		glutSolidCone ( 0.6, 1.0, 40, 40 )
	elif object == 4:
		glutSolidTorus ( 0.25, 0.5, 40, 40 )
	elif object == 5:
		glutSolidDodecahedron ()
	elif object == 6:
		glutSolidOctahedron ()
	elif object == 7:
		glutSolidTetrahedron ()
	elif object == 8:
		glutSolidIcosahedron ()
	
	program.unbind ()
	
	glPopMatrix     ()
	glutSwapBuffers ()

def keyPressed ( *args ):
	global	rot, rot2, object
	if args [0] == '\033':
		sys.exit ()
	
	if args [0] == GLUT_KEY_RIGHT:
		rot += 5
	elif args [0] == GLUT_KEY_LEFT:
		rot -= 5
	elif args [0] == GLUT_KEY_UP:
		rot2 -= 2
	elif args [0] == GLUT_KEY_DOWN:
		rot2 += 2
	
	if args [0] == '+':
		object += 1
		if object > 6:
			object = 0
	
	if args [0] == '-':
		object -= 1
		if object < 0:
			object = 6

def specialKey ( key, x, y ):
	global	rot, rot2, object
	
	if key == GLUT_KEY_UP:
		rot2 -= 5.0
	elif key == GLUT_KEY_DOWN:
		rot2 += 5.0
	elif key == GLUT_KEY_LEFT:
		rot -= 5.0
	elif key == GLUT_KEY_RIGHT:
		rot += 5.0
	else:
		return
	
	glutPostRedisplay()

def animate ():
	global lastTime, program, angle
	
	time = 0.001 * glutGet ( GLUT_ELAPSED_TIME );
	
	if useRotation:
		angle += 2 * (time - lastTime)
	
	lastTime = time

	program.bind       ()
	program.setUniform ( "eyePos", [0, 0, 0, 1] )
	program.unbind     ()

	glutPostRedisplay ()

def main ():
	global decalMap, stoneMap, teapotMap, fringeMap, reflectionMap, program
	
	glutInit               ( sys.argv )
	glutInitDisplayMode    ( GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH )
	glutInitWindowSize     ( 500, 500 )
	glutInitWindowPosition ( 0, 0 )
	
	glutCreateWindow ( "PyOpenGL glass example" )
	glutDisplayFunc  ( display    )
	glutIdleFunc     ( animate    )
	glutReshapeFunc  ( reshape    )
	glutKeyboardFunc ( keyPressed )
	glutSpecialFunc  ( specialKey )
	init             ()
	
	if not GlslProgram.isSupported ():
		print "GLSL not supported"
		sys.exit ( 1 )
		
	vSource = open ( "glass-2.vsh" ).read ()
	fSource = open ( "glass-2.fsh" ).read ()
	program = GlslProgram.GlslProgram ()
	program.loadShaders ( vSource, fSource );
	
	reflectionMap = loadCubemap ( ( "cm_left.tga", "cm_right.tga", "cm_top.tga", "cm_bottom.tga", "cm_back.tga", "cm_front.tga" ), "../../Textures/Cubemaps" )
	fringeMap     = loadTexture ( "fringe.png" )
	decalMap      = loadTexture ( "../../Textures/oak.bmp" )
	stoneMap      = loadTexture ( "../../Textures/block.bmp" )
	teapotMap     = loadTexture ( "../../Textures/Oxidated.jpg" )
	
	print program.getLog ()
	
	program.bind       ()
	program.setTexture ( "envMap", 0 )
	program.unbind     ()
	
	glutMainLoop()

print "Hit ESC key to quit."
main()
    	
