#
# Examples for PyOpenGL (and PIL) and GLSL 
# Python port of sepia example from "Shaders" book
#
# Author: Alex V. Boreskoff
#

from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
from OpenGL.GL.ARB.texture_cube_map import *
import Image 				# PIL
import sys
import math
import GlslProgram
import FrameBuffer
from utils import *

decalMap      = None
stoneMap      = None
teapotMap     = None
screenMap     = None
angle         = 0.0
rot           = 0.0
angle         = 0.0
useFilter     = 1
program       = None
buffer        = None

def displayBoxes ():
	global angle, rot, stoneMap, decalMap, teapotMap
	glMatrixMode ( GL_MODELVIEW )
	glPushMatrix ()
	glRotatef    ( rot, 0, 0, 1 )
	
	drawBoxCull  ( -5, -5, 0, 10, 10, 3, stoneMap, None )
	drawBoxCull  ( 3, 2, 0.5, 1,  2,  2, decalMap, 1 )
	
	glBindTexture   ( GL_TEXTURE_2D, teapotMap )
	glTranslatef    ( 0.2, 1, 1.5 )
	glRotatef       ( angle * 45.3, 1, 0, 0 )
	glRotatef       ( angle * 57.2, 0, 1, 0 )
	glutSolidTeapot ( 0.3 )
	glPopMatrix     ()

def startOrtho ():
	glMatrixMode   ( GL_PROJECTION )                    # select the projection matrix
	glPushMatrix   ()                                   # store the projection matrix
	glLoadIdentity ()                                   # reset the projection matrix
														# set up an ortho screen
	glOrtho        ( 0, 512, 0, 512, -1, 1 )
	glMatrixMode   ( GL_MODELVIEW )                     # select the modelview matrix
	glPushMatrix   ()                                   # store the modelview matrix
	glLoadIdentity ()                                   # reset the modelview matrix

def endOrtho ():
	glMatrixMode ( GL_PROJECTION )                      # select the projection matrix
	glPopMatrix  ()                                     # restore the old projection matrix
	glMatrixMode ( GL_MODELVIEW )                       # select the modelview matrix
	glPopMatrix  ()                                     # restore the old projection matrix

def init ():
	glClearColor ( 0.0, 0.0, 0.0, 0.0 )
	glClearDepth ( 1.0 )				
	glDepthFunc  ( GL_LEQUAL )
	glEnable     ( GL_DEPTH_TEST )
	glEnable     ( GL_TEXTURE_2D )
	glHint       ( GL_POLYGON_SMOOTH_HINT,         GL_NICEST )
	glHint       ( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST )

def reshape ( width, height ):
	glViewport     ( 0, 0, width, height )
	glMatrixMode   ( GL_PROJECTION )
	glLoadIdentity ()
	gluPerspective ( 60.0, float(width)/float (height), 1.0, 60.0 )
	glMatrixMode   ( GL_MODELVIEW )
	glLoadIdentity ()
	gluLookAt      ( -0.5, -0.5, 1.5, 3.0, 3.0, 1.0, 0.0, 0.0, 1.0 )

def display ():
	global screenMap, program, useFilter
	
	renderToBuffer ()
	
	glClear       ( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT )
	startOrtho    ()
	glBindTexture ( GL_TEXTURE_2D, screenMap )
	
	if useFilter:
		program.bind ()
	
	glBegin      ( GL_QUADS )
	glTexCoord2f ( 0, 0 )
	glVertex2f   ( 0, 0 )
	glTexCoord2f ( 1,   0 )
	glVertex2f   ( 512, 0 )
	glTexCoord2f ( 1, 1 )
	glVertex2f   ( 512, 512 )
	glTexCoord2f ( 0, 1 )
	glVertex2f   ( 0, 512 )
	glEnd        ()
	
	if useFilter:
		program.unbind ()
	
	endOrtho        ()
	glutSwapBuffers ()

def keyPressed ( *args ):
	global	useFilter
	
	if args [0] == '\033' or args [0] == 'q' or args [0] == 'Q':
		sys.exit ()
	
	if args [0] == 'f' or args [0] == 'F':
		useFilter = not useFilter

def specialKey ( key, x, y ):
	global	rot
	
	if key == GLUT_KEY_LEFT:
		rot -= 5.0
	elif key == GLUT_KEY_RIGHT:
		rot += 5.0
	else:
		return
	
	glutPostRedisplay()

def animate ():
	global angle, program
	angle  = 0.004 * glutGet ( GLUT_ELAPSED_TIME )
	
	glutPostRedisplay ()

def renderToBuffer ():
	global buffer
	glBindTexture ( GL_TEXTURE_2D, 0 )
	
	buffer.bind ()
	
	glClearColor ( 0, 0, 1, 1 )
	glClear      ( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT )
	
	reshape ( buffer.width, buffer.height )
	
	displayBoxes ()
	
	buffer.unbind ()

def main ():
	global decalMap, stoneMap, teapotMap, screenMap, program, buffer
	
	glutInit               ( sys.argv )
	glutInitDisplayMode    ( GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH )
	glutInitWindowSize     ( 512, 512 )
	glutInitWindowPosition ( 0, 0 )
	
	glutCreateWindow ( "PyOpenGL sepia example" )
	glutDisplayFunc  ( display    )
	glutIdleFunc     ( animate    )
	glutReshapeFunc  ( reshape    )
	glutKeyboardFunc ( keyPressed )
	glutSpecialFunc  ( specialKey )
	init             ()

	if not GlslProgram.isSupported ():
		print "GLSL not supported"
		sys.exit ( 1 )
		
	vSource = open ( "sepia.vsh" ).read ()
	fSource = open ( "sepia.fsh" ).read ()
	program = GlslProgram.GlslProgram ()
	program.loadShaders ( vSource, fSource );
	
	decalMap  = loadTexture ( "../../Textures/oak.bmp" )
	stoneMap  = loadTexture ( "../../Textures/block.bmp" )
	teapotMap = loadTexture ( "../../Textures/Oxidated.jpg" )
	
	buffer    = FrameBuffer.FrameBuffer ( 512, 512, 4 )
	screenMap = buffer.createColorTexture ();
	buffer.create ()
	buffer.bind   ()
	buffer.attachColorTexture ( GL_TEXTURE_2D, screenMap )

	program.bind       ()
	program.setTexture ( "mainTex", 0 )
	program.unbind     ()
	
	glutMainLoop()

print "Hit ESC key to quit."
main()
    	
