# 
# simple OpengL utils
#

from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
from OpenGL.GL.ARB.texture_cube_map import *
import Image 		

def drawBox ( x1, x2, y1, y2, z1, z2, texture ):
	glBindTexture ( GL_TEXTURE_2D, texture )
	glBegin       ( GL_POLYGON )         				# front face
	glNormal3f    ( 0.0, 0.0, 1.0 )
	glTexCoord2f  ( 0, 0 )
	glVertex3f    ( x1, y1, z2 )
	glTexCoord2f  ( 1, 0 )
	glVertex3f    ( x2, y1, z2 )
	glTexCoord2f  ( 1, 1 )
	glVertex3f    ( x2, y2, z2 )
	glTexCoord2f  ( 0, 1 )
	glVertex3f    ( x1, y2, z2 )
	glEnd         ()
	
	glBegin      ( GL_POLYGON )         				# back face
	glNormal3f   ( 0.0, 0.0, -1.0 )
	glTexCoord2f ( 1, 0 )
	glVertex3f   ( x2, y1, z1 )
	glTexCoord2f ( 0, 0 )
	glVertex3f   ( x1, y1, z1 )
	glTexCoord2f ( 0, 1 )
	glVertex3f   ( x1, y2, z1 )
	glTexCoord2f ( 1, 1 )
	glVertex3f   ( x2, y2, z1 )
	glEnd        ()
	
	glBegin      ( GL_POLYGON )         				# left face
	glNormal3f   ( -1.0, 0.0, 0.0 )
	glTexCoord2f ( 0, 0 )
	glVertex3f   ( x1, y1, z1 )
	glTexCoord2f ( 0, 1 )
	glVertex3f   ( x1, y1, z2 )
	glTexCoord2f ( 1, 1 )
	glVertex3f   ( x1, y2, z2 )
	glTexCoord2f ( 1, 0 )
	glVertex3f   ( x1, y2, z1 )
	glEnd        ()
	
	glBegin      ( GL_POLYGON )         				#  right face
	glNormal3f   ( 1.0, 0.0, 0.0 )
	glTexCoord2f ( 0, 1 )
	glVertex3f   ( x2, y1, z2 )
	glTexCoord2f ( 0, 0 )
	glVertex3f   ( x2, y1, z1 )
	glTexCoord2f ( 1, 0 )
	glVertex3f   ( x2, y2, z1 )
	glTexCoord2f ( 1, 1 )
	glVertex3f   ( x2, y2, z2 )
	glEnd        ()
	
	glBegin      ( GL_POLYGON )         		# top face
	glNormal3f   ( 0.0, 1.0, 0.0 )
	glTexCoord2f ( 0, 1 )
	glVertex3f   ( x1, y2, z2 )
	glTexCoord2f ( 1, 1 )
	glVertex3f   ( x2, y2, z2 )
	glTexCoord2f ( 1, 0 )
	glVertex3f   ( x2, y2, z1 )
	glTexCoord2f ( 0, 0 )
	glVertex3f   ( x1, y2, z1 )
	glEnd        ()
	
	glBegin      ( GL_POLYGON )         		#  bottom face
	glNormal3f   ( 0.0, -1.0, 0.0 )
	glTexCoord2f ( 1, 1 )
	glVertex3f   ( x2, y1, z2 )
	glTexCoord2f ( 1, 0 )
	glVertex3f   ( x1, y1, z2 )
	glTexCoord2f ( 0, 0 )
	glVertex3f   ( x1, y1, z1 )
	glTexCoord2f ( 0, 1 )
	glVertex3f   ( x2, y1, z1 )
	glEnd        ()

def drawBoxCull ( x1, y1, z1, sx, sy, sz, texture, cull = None ):
	x2 = x1 + sx
	y2 = y1 + sy
	z2 = z1 + sz
	
	if cull:
		glCullFace ( GL_BACK )
		glEnable   ( GL_CULL_FACE )
	else:
		glDisable ( GL_CULL_FACE )
	
	glBindTexture ( GL_TEXTURE_2D, texture )
	glBegin       ( GL_POLYGON )         				# front face
	glNormal3f    ( 0.0, 0.0, 1.0 )
	glTexCoord2f  ( 0, 0 )
	glVertex3f    ( x1, y1, z2 )
	glTexCoord2f  ( 1, 0 )
	glVertex3f    ( x2, y1, z2 )
	glTexCoord2f  ( 1, 1 )
	glVertex3f    ( x2, y2, z2 )
	glTexCoord2f  ( 0, 1 )
	glVertex3f    ( x1, y2, z2 )
	glEnd         ()
	
	glBegin      ( GL_POLYGON )         				# back face
	glNormal3f   ( 0.0, 0.0, -1.0 )
	glTexCoord2f ( 1, 0 )
	glVertex3f   ( x2, y1, z1 )
	glTexCoord2f ( 0, 0 )
	glVertex3f   ( x1, y1, z1 )
	glTexCoord2f ( 0, 1 )
	glVertex3f   ( x1, y2, z1 )
	glTexCoord2f ( 1, 1 )
	glVertex3f   ( x2, y2, z1 )
	glEnd        ()
	
	glBegin      ( GL_POLYGON )         				# left face
	glNormal3f   ( -1.0, 0.0, 0.0 )
	glTexCoord2f ( 0, 0 )
	glVertex3f   ( x1, y1, z1 )
	glTexCoord2f ( 0, 1 )
	glVertex3f   ( x1, y1, z2 )
	glTexCoord2f ( 1, 1 )
	glVertex3f   ( x1, y2, z2 )
	glTexCoord2f ( 1, 0 )
	glVertex3f   ( x1, y2, z1 )
	glEnd        ()
	
	glBegin      ( GL_POLYGON )         				#  right face
	glNormal3f   ( 1.0, 0.0, 0.0 )
	glTexCoord2f ( 0, 1 )
	glVertex3f   ( x2, y1, z2 )
	glTexCoord2f ( 0, 0 )
	glVertex3f   ( x2, y1, z1 )
	glTexCoord2f ( 1, 0 )
	glVertex3f   ( x2, y2, z1 )
	glTexCoord2f ( 1, 1 )
	glVertex3f   ( x2, y2, z2 )
	glEnd        ()
	
	glBegin      ( GL_POLYGON )         		# top face
	glNormal3f   ( 0.0, 1.0, 0.0 )
	glTexCoord2f ( 0, 1 )
	glVertex3f   ( x1, y2, z2 )
	glTexCoord2f ( 1, 1 )
	glVertex3f   ( x2, y2, z2 )
	glTexCoord2f ( 1, 0 )
	glVertex3f   ( x2, y2, z1 )
	glTexCoord2f ( 0, 0 )
	glVertex3f   ( x1, y2, z1 )
	glEnd        ()
	
	glBegin      ( GL_POLYGON )         		#  bottom face
	glNormal3f   ( 0.0, -1.0, 0.0 )
	glTexCoord2f ( 1, 1 )
	glVertex3f   ( x2, y1, z2 )
	glTexCoord2f ( 1, 0 )
	glVertex3f   ( x1, y1, z2 )
	glTexCoord2f ( 0, 0 )
	glVertex3f   ( x1, y1, z1 )
	glTexCoord2f ( 1, 0 )
	glVertex3f   ( x2, y1, z1 )
	glEnd        ()
	
	if cull:
		glDisable ( GL_CULL_FACE )


def	drawCubicMappedBox ( size, texture ):
	glBindTexture ( GL_TEXTURE_CUBE_MAP_ARB, texture )
	glEnable      ( GL_TEXTURE_CUBE_MAP_ARB )
	
	glBegin ( GL_QUADS )
	glNormal3f   ( 0, 0, 1 )		
	glTexCoord3f ( -size [0], -size [1], size [2] )
	glVertex3f   ( -size [0], -size [1], size [2] )
	glTexCoord3f (  size [0], -size [1], size [2] )
	glVertex3f   (  size [0], -size [1], size [2] )
	glTexCoord3f (  size [0],  size [1], size [2] )
	glVertex3f   (  size [0],  size [1], size [2] )
	glTexCoord3f ( -size [0],  size [1], size [2] )
	glVertex3f   ( -size [0],  size [1], size [2] )
	
	glNormal3f   ( 0, 0, -1 )
	glTexCoord3f (  size [0], -size [1], -size [2] )
	glVertex3f   (  size [0], -size [1], -size [2] )
	glTexCoord3f ( -size [0], -size [1], -size [2] )
	glVertex3f   ( -size [0], -size [1], -size [2] )
	glTexCoord3f ( -size [0],  size [1], -size [2] )
	glVertex3f   ( -size [0],  size [1], -size [2] )
	glTexCoord3f (  size [0],  size [1], -size [2] )
	glVertex3f   (  size [0],  size [1], -size [2] )
	
	glNormal3f   ( -1, 0, 0 )
	glTexCoord3f ( -size [0], -size [1], -size [2] )
	glVertex3f   ( -size [0], -size [1], -size [2] )
	glTexCoord3f ( -size [0], -size [1],  size [2] )
	glVertex3f   ( -size [0], -size [1],  size [2] )
	glTexCoord3f ( -size [0],  size [1],  size [2] )
	glVertex3f   ( -size [0],  size [1],  size [2] )
	glTexCoord3f ( -size [0],  size [1], -size [2] )
	glVertex3f   ( -size [0],  size [1], -size [2] )
	
	glNormal3f   ( 1, 0, 0 )
	glTexCoord3f ( size [0], -size [1],  size [2] )
	glVertex3f   ( size [0], -size [1],  size [2] )
	glTexCoord3f ( size [0], -size [1], -size [2] )
	glVertex3f   ( size [0], -size [1], -size [2] )
	glTexCoord3f ( size [0],  size [1], -size [2] )
	glVertex3f   ( size [0],  size [1], -size [2] )
	glTexCoord3f ( size [0],  size [1],  size [2] )
	glVertex3f   ( size [0],  size [1],  size [2] )
	
	glNormal3f   ( 0, 1, 0 )
	glTexCoord3f ( -size [0], size [1],  size [2] )
	glVertex3f   ( -size [0], size [1],  size [2] )
	glTexCoord3f (  size [0], size [1],  size [2] )
	glVertex3f   (  size [0], size [1],  size [2] )
	glTexCoord3f (  size [0], size [1], -size [2] )
	glVertex3f   (  size [0], size [1], -size [2] )
	glTexCoord3f ( -size [0], size [1], -size [2] )
	glVertex3f   ( -size [0], size [1], -size [2] )
	
	glNormal3f   ( 0, -1, 0 )
	glTexCoord3f (  size [0], -size [1],  size [2] )
	glVertex3f   (  size [0], -size [1],  size [2] )
	glTexCoord3f ( -size [0], -size [1],  size [2] )
	glVertex3f   ( -size [0], -size [1],  size [2] )
	glTexCoord3f ( -size [0], -size [1], -size [2] )
	glVertex3f   ( -size [0], -size [1], -size [2] )
	glTexCoord3f (  size [0], -size [1], -size [2] )
	glVertex3f   (  size [0], -size [1], -size [2] )
	glEnd ()

def loadCubemap ( faces, path = "" ):
	texture = glGenTextures ( 1 )
	target  = GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB
	glBindTexture     ( GL_TEXTURE_CUBE_MAP_ARB, texture )   # 2d texture (x and y size)
	glPixelStorei     ( GL_UNPACK_ALIGNMENT,1 )
	glTexParameteri   ( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,     GL_REPEAT )
	glTexParameteri   ( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,     GL_REPEAT )
	glTexParameteri   ( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR )
	glTexParameteri   ( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR )
	glEnable          ( GL_TEXTURE_CUBE_MAP_ARB )
	
	for f in faces:
		if path != "":
			file = path + "/" + f
		else:
			file = f
		
		image  = Image.open ( file )
		width  = image.size [0]
		height = image.size [1]
		image  = image.tostring ( "raw", "RGBX", 0, -1 )
		gluBuild2DMipmaps ( target, 3, width, height, GL_RGBA, GL_UNSIGNED_BYTE, image )
		target = target + 1
	
	return texture


def loadTexture ( fileName ):
	image  = Image.open ( fileName )
	width  = image.size [0]
	height = image.size [1]
	image  = image.tostring ( "raw", "RGBX", 0, -1 )
	texture = glGenTextures ( 1 )
	
	glBindTexture     ( GL_TEXTURE_2D, texture )   # 2d texture (x and y size)
	glPixelStorei     ( GL_UNPACK_ALIGNMENT,1 )
	glTexParameterf   ( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT )
	glTexParameterf   ( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT )
	glTexParameteri   ( GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR )
	glTexParameteri   ( GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR )
	gluBuild2DMipmaps ( GL_TEXTURE_2D, 3, width, height, GL_RGBA, GL_UNSIGNED_BYTE, image )
	
	return texture
