//
// Wrapper of FrameBuffer class for Python
//

#include	<Python.h>
#include	"libExt.h"
#include	"FrameBuffer.h"
#include	"Data.h"
#include	"Vector4D.h"
#include	"Vector3D.h"
#include	"Vector2D.h"

#ifdef	_WIN32
	#define	DLL_EXPORT	__declspec(dllexport) 
#else
	#define	DLL_EXPORT
#endif

struct  FrameBufferObject
{
	PyObject_HEAD
	
	FrameBuffer	* fb;
};

#define FrameBufferObject_Check(v)	((v)->ob_type == &FrameBuffer_Type)

static PyObject * create                 ( FrameBufferObject * self, PyObject * args );
static PyObject * isOk                   ( FrameBufferObject * self, PyObject * args );
static PyObject * bindFunc               ( FrameBufferObject * self, PyObject * args );
static PyObject * unbind                 ( FrameBufferObject * self, PyObject * args );
static PyObject * attachColorTexture     ( FrameBufferObject * self, PyObject * args );
static PyObject * attachDepthTexture     ( FrameBufferObject * self, PyObject * args );
static PyObject * detachColorTexture     ( FrameBufferObject * self, PyObject * args );
static PyObject * detachDepthTexture     ( FrameBufferObject * self, PyObject * args );
static PyObject * createColorTexture     ( FrameBufferObject * self, PyObject * args );
static PyObject * createColorRectTexture ( FrameBufferObject * self, PyObject * args );
static PyObject * buildMipmaps           ( FrameBufferObject * self, PyObject * args );
static PyObject * isSupported            ( PyObject * self, PyObject * args );

static PyObject * getattr        ( FrameBufferObject * self, char * name );
static PyObject * repr           ( FrameBufferObject * self );
static void       FrameBuffer_dealloc ( FrameBufferObject * self );

static PyTypeObject FrameBuffer_Type = 
{
	PyObject_HEAD_INIT(NULL)
	0,									/*ob_size*/
	"FrameBuffer",						/*tp_name*/
	sizeof(FrameBufferObject),			/*tp_basicsize*/
	0,									/*tp_itemsize*/
										/* methods */
	(destructor)FrameBuffer_dealloc, 	/*tp_dealloc*/
	0,									/*tp_print*/
	(getattrfunc)getattr, 				/*tp_getattr*/
	0, 									/*tp_setattr*/
	0,									/*tp_compare*/
	(reprfunc)repr,						/*tp_repr*/
	0,									/*tp_as_number*/
	0,									/*tp_as_sequence*/
	0,									/*tp_as_mapping*/
	0,									/*tp_hash*/
};

static PyMethodDef FrameBuffer_Module_methods [] = 
{
	{ "isSupported", (PyCFunction)isSupported, METH_VARARGS },
	{ NULL,		NULL}		/* sentinel */
};

static PyMethodDef FrameBuffer_methods [] = 
{
	{ "create",	 				(PyCFunction)create,      				METH_VARARGS, "" },
	{ "isOk",     				(PyCFunction)isOk, 	  					METH_VARARGS, "" },
	{ "bind",     			    (PyCFunction)bindFunc, 	 	 			METH_VARARGS, "" },
	{ "unbind",     			(PyCFunction)unbind, 	  				METH_VARARGS, "" },
	{ "attachColorTexture",	    (PyCFunction)attachColorTexture,		METH_VARARGS, "" },
	{ "attachDepthTexture",	    (PyCFunction)attachDepthTexture,		METH_VARARGS, "" },
	{ "detachColorTexture",	    (PyCFunction)detachColorTexture,		METH_VARARGS, "" },
	{ "detachDepthTexture",	    (PyCFunction)detachDepthTexture,		METH_VARARGS, "" },
	{ "createColorTexture",	    (PyCFunction)createColorTexture,		METH_VARARGS, "" },
	{ "createColorRectTexture",	(PyCFunction)createColorRectTexture,	METH_VARARGS, "" },
	{ "buildMipmaps",			(PyCFunction)buildMipmaps,	  			METH_VARARGS, "" },
	{ NULL,		NULL}	
};

static PyObject * FrameBuffer_new ( PyTypeObject * type, PyObject * args, PyObject * kwds )
{
	int	width, height, flags = 0;
	
	if ( !PyArg_ParseTuple ( args, "ii|i", &width, &height, &flags ) )
		return NULL;
	
	initExtensions ();
	
	FrameBufferObject * object = PyObject_NEW ( FrameBufferObject, &FrameBuffer_Type );
	
	if ( object == NULL )
		return NULL;
		
	object -> fb = new FrameBuffer ( width, height, flags );
	
	return (PyObject *) object;
}

static PyObject * isSupported ( PyObject * self, PyObject * args )
{
	initExtensions ();
	
	return Py_BuildValue ( "i", FrameBuffer :: isSupported () );
}

static void FrameBuffer_dealloc ( FrameBufferObject * self )
{
	delete ((FrameBufferObject *) self) -> fb;
	
	PyObject_DEL ( self );
}

static PyObject * create ( FrameBufferObject * self, PyObject * args )
{
	return Py_BuildValue ( "i",  self -> fb -> create () );
}

static PyObject * isOk ( FrameBufferObject * self, PyObject * args )
{
	return Py_BuildValue ( "i", self -> fb -> isOk () );
}

static PyObject * bindFunc ( FrameBufferObject * self, PyObject * args )
{
	self -> fb -> bind ();
	
	return Py_BuildValue ( "i", true );
}

static PyObject * unbind ( FrameBufferObject * self, PyObject * args )
{
	self -> fb -> unbind ();
	
	return Py_BuildValue ( "i", true );
}

static PyObject * attachColorTexture ( FrameBufferObject * self, PyObject * args )
{
	unsigned	target, texId, no = 0;
	
	if ( !PyArg_ParseTuple ( args, "II|I", &target, &texId, &no ) )
		return NULL;
		
	return Py_BuildValue ( "i", self -> fb -> attachColorTexture ( target, texId, no ) );
}

static PyObject * attachDepthTexture ( FrameBufferObject * self, PyObject * args )
{
	unsigned	target, texId;
	
	if ( !PyArg_ParseTuple ( args, "II", &target, &texId ) )
		return NULL;
		
	return Py_BuildValue ( "i", self -> fb -> attachDepthTexture ( target, texId ) );
}

static PyObject * detachColorTexture ( FrameBufferObject * self, PyObject * args )
{
	unsigned	target;
	
	if ( !PyArg_ParseTuple ( args, "I", &target ) )
		return NULL;
		
	return Py_BuildValue ( "i", self -> fb -> detachColorTexture ( target ) );
}

static PyObject * detachDepthTexture ( FrameBufferObject * self, PyObject * args )
{
	unsigned	target;
	
	if ( !PyArg_ParseTuple ( args, "I", &target ) )
		return NULL;
		
	return Py_BuildValue ( "i", self -> fb -> detachDepthTexture ( target ) );
}

static PyObject * createColorTexture ( FrameBufferObject * self, PyObject * args )
{
	unsigned	format    = GL_RGBA;
	unsigned	intFormat = GL_RGBA8;
	unsigned	clamp     = GL_REPEAT;
	int			filter    = FrameBuffer :: filterLinear;
	
	if ( !PyArg_ParseTuple ( args, "|IIII", &format, &intFormat, &clamp, &filter ) )
		return NULL;
		
	return Py_BuildValue ( "i", self -> fb -> createColorTexture ( format, intFormat, clamp, filter ) );
}

static PyObject * createColorRectTexture ( FrameBufferObject * self, PyObject * args )
{
	unsigned	format    = GL_RGBA;
	unsigned	intFormat = GL_RGBA8;
	
	if ( !PyArg_ParseTuple ( args, "|II", &format, &intFormat ) )
		return NULL;
		
	return Py_BuildValue ( "i", self -> fb -> createColorRectTexture ( format, intFormat ) );
}

static PyObject * buildMipmaps ( FrameBufferObject * self, PyObject * args )
{
	self -> fb -> buildMipmaps ();
	
	return Py_BuildValue ( "i", true );
}

static PyObject * getattr ( FrameBufferObject * self, char * name )
{
    PyObject * res = Py_FindMethod ( FrameBuffer_methods, (PyObject*) self, name );

    if ( res )
        return res;

    PyErr_Clear ();
	
    if ( !strcmp ( name, "width" ) )							// attributes
        return PyInt_FromLong ( self -> fb -> getWidth () );
	else
    if ( !strcmp ( name, "height" ) )
        return PyInt_FromLong ( self -> fb -> getHeight () );
	else
    if ( !strcmp ( name, "hasStencil" ) )
        return PyInt_FromLong ( self -> fb -> hasStencil () );
	else
    if ( !strcmp ( name, "hasDepth" ) )
        return PyInt_FromLong ( self -> fb -> hasDepth () );
	
    return NULL;
}

static PyObject * repr ( FrameBufferObject * self )
{
	char buf [128];
	
	sprintf ( buf, "FrameBuffer - %d x %d pixels", self -> fb -> getWidth (), self -> fb -> getHeight () );
	
	return PyString_FromString ( buf );
}

extern	"C" void	DLL_EXPORT initFrameBuffer ()
{
	FrameBuffer_Type.ob_type   = &PyType_Type;
	FrameBuffer_Type.tp_new    = FrameBuffer_new;
	FrameBuffer_Type.tp_flags |= Py_TPFLAGS_CHECKTYPES;
	
						// Create the module and add the functions
	PyObject * m = Py_InitModule ( "FrameBuffer", FrameBuffer_Module_methods );
	
	Py_INCREF          ( &FrameBuffer_Type );
    PyModule_AddObject ( m, "FrameBuffer", (PyObject *)&FrameBuffer_Type );
}
	

