//
// Wrapper of GlslProgram class for Python
//

#include	<Python.h>
#include	"GlslProgram.h"
#include	"libExt.h"
#include	"Data.h"
#include	"Vector4D.h"
#include	"Vector3D.h"
#include	"Vector2D.h"

#ifdef	_WIN32
	#define	DLL_EXPORT	__declspec(dllexport) 
#else
	#define	DLL_EXPORT
#endif

struct  GlslProgramObject
{
	PyObject_HEAD
	
	GlslProgram	* p;
};

#define GlslProgramObject_Check(v)	((v)->ob_type == &GlslProgram_Type)

static PyObject * loadShaders         ( GlslProgramObject * self, PyObject * args );
static PyObject * clear               ( GlslProgramObject * self, PyObject * args );
static PyObject * getLog              ( GlslProgramObject * self, PyObject * args );
static PyObject * isOk                ( GlslProgramObject * self, PyObject * args );
static PyObject * bindFunc            ( GlslProgramObject * self, PyObject * args );
static PyObject * unbind              ( GlslProgramObject * self, PyObject * args );
static PyObject * isSupported         ( PyObject          * self, PyObject * args );
static PyObject * setUniform          ( GlslProgramObject * self, PyObject * args );
static PyObject * setTexture          ( GlslProgramObject * self, PyObject * args );
static PyObject * setUniformMatrix    ( GlslProgramObject * self, PyObject * args );

static PyObject * getattr             ( GlslProgramObject * self, char * name );
static PyObject * repr                ( GlslProgramObject * self );
static void       GlslProgram_dealloc ( GlslProgramObject * self );

static PyTypeObject GlslProgram_Type = 
{
	PyObject_HEAD_INIT(NULL)
	0,									/*ob_size*/
	"GlslProgram",						/*tp_name*/
	sizeof(GlslProgramObject),			/*tp_basicsize*/
	0,									/*tp_itemsize*/
										/* methods */
	(destructor)GlslProgram_dealloc, 	/*tp_dealloc*/
	0,									/*tp_print*/
	(getattrfunc)getattr, 				/*tp_getattr*/
	0, 									/*tp_setattr*/
	0,									/*tp_compare*/
	(reprfunc)repr,						/*tp_repr*/
	0,									/*tp_as_number*/
	0,									/*tp_as_sequence*/
	0,									/*tp_as_mapping*/
	0,									/*tp_hash*/
};

static PyMethodDef GlslProgram_Module_methods [] = 
{
	{ "isSupported", (PyCFunction)isSupported, METH_VARARGS },
	{ NULL,		NULL}		/* sentinel */
};

static PyMethodDef GlslProgram_methods [] = 
{
	{ "loadShaders",	 		(PyCFunction)loadShaders,      		METH_VARARGS, "" },
	{ "clear",	    			(PyCFunction) clear,	      		METH_VARARGS, "" },
	{ "getLog",					(PyCFunction) getLog,	  			METH_VARARGS, "" },
	{ "isOk",     				(PyCFunction) isOk, 	  			METH_VARARGS, "" },
	{ "bind",     			    (PyCFunction) bindFunc, 	  		METH_VARARGS, "" },
	{ "unbind",     			(PyCFunction) unbind, 	  			METH_VARARGS, "" },
	{ "setUniform",     		(PyCFunction) setUniform, 	  		METH_VARARGS, "" },
	{ "setUniformMatrix",     	(PyCFunction) setUniformMatrix, 	METH_VARARGS, "" },
	{ "setTexture",     		(PyCFunction) setTexture, 	  		METH_VARARGS, "" },
	{ NULL,		NULL}		/* sentinel */
};

static	int	getArray ( PyObject * array, float * buf, int maxSize )
{
	int	size = -1;
	
	if ( PyTuple_Check ( array ) )
	{	
		size = PyTuple_Size ( array );
		
		for ( int i = 0; i < size && i < maxSize; i++ )
		{
			PyObject * item = PyTuple_GetItem ( array, i );
			
			if ( item == NULL )
				return -1;
				
			buf [i] = (float) PyFloat_AsDouble ( item );
		}
	}
	else
	if ( PyList_Check ( array ) )
	{	
		size = PyList_Size ( array );

		for ( int i = 0; i < size && i < maxSize; i++ )
		{
			PyObject * item = PyList_GetItem ( array, i );
			
			if ( item == NULL )
				return -1;
				
			buf [i] = (float) PyFloat_AsDouble ( item );
		}
	}

	return size;
}

static PyObject * GlslProgram_new ( PyTypeObject * type, PyObject * args, PyObject * kwds )
{
	initExtensions ();
	
	GlslProgramObject * object = PyObject_NEW ( GlslProgramObject, &GlslProgram_Type );
	
	if ( object == NULL )
		return NULL;
		
	object -> p = new GlslProgram ();
	
	return (PyObject *) object;
}

static PyObject * isSupported ( PyObject * self, PyObject * args )
{
	initExtensions ();

	return Py_BuildValue ( "i", GlslProgram :: isSupported () );
}

static void GlslProgram_dealloc ( GlslProgramObject * self )
{
	delete ((GlslProgramObject *) self) -> p;
	
	PyObject_DEL ( self );
}

static PyObject * loadShaders ( GlslProgramObject * self, PyObject * args )
{
	char * vs;
	char * fs;
	
	if ( !PyArg_ParseTuple ( args, "ss", &vs, &fs ) )
        return NULL;

	Data	vertexData   ( vs, strlen ( vs ) + 1 );
	Data	fragmentData ( fs, strlen ( fs ) + 1 );
	bool	res = self -> p -> loadShaders ( &vertexData, &fragmentData );
	
	return Py_BuildValue ( "i",  (int) res );
}

static PyObject * getLog ( GlslProgramObject * self, PyObject * args )
{
	return PyString_FromString ( self -> p -> getLog ().c_str () );
}

static PyObject * isOk ( GlslProgramObject * self, PyObject * args )
{
	return Py_BuildValue ( "i", self -> p -> isOk () );
}

static PyObject * bindFunc ( GlslProgramObject * self, PyObject * args )
{
	self -> p -> bind ();
	
	return Py_BuildValue ( "i", true );
}

static PyObject * unbind ( GlslProgramObject * self, PyObject * args )
{
	self -> p -> unbind ();
	
	return Py_BuildValue ( "i", true );
}

static PyObject * clear ( GlslProgramObject * self, PyObject * args )
{
	self -> p -> clear ();
	
	return Py_BuildValue ( "i", true );
}

static PyObject * setUniform  ( GlslProgramObject * self, PyObject * args )
{
	char     * name;
	PyObject * tuple;
	float	   v [4];

	if ( !PyArg_ParseTuple ( args, "sO", &name, &tuple ) )
		return NULL;

	int	size = getArray ( tuple, v, 4 );
	
	if ( size < 1 || size > 4 )
		return NULL;
		
	if ( size == 4 )
		self -> p -> setUniformVector ( name, Vector4D ( v [0], v [1], v [2], v [3] ) );
	else
	if ( size == 3 )
		self -> p -> setUniformVector ( name, Vector3D ( v [0], v [1], v [2] ) );
	else
	if ( size == 2 )
		self -> p -> setUniformVector ( name, Vector2D ( v [0], v [1] ) );
	else
		self -> p -> setUniformFloat ( name, v [0] );

	return Py_BuildValue ( "i", true );
}

static PyObject * setUniformMatrix ( GlslProgramObject * self, PyObject * args )
{
	char     * name;
	PyObject * tuple;
	float	   m [16];
	
	if ( !PyArg_ParseTuple ( args, "sO", &name, &tuple ) )
		return NULL;
	
	int	size = getArray ( tuple, m, 16 );
	
	if ( size != 16 )
		return NULL;
		
	self -> p -> setUniformMatrix ( name, m );

	return Py_BuildValue ( "i", true );
}

static PyObject * setTexture  ( GlslProgramObject * self, PyObject * args )
{
	char  * name;
	int		unit;
	
	if ( !PyArg_ParseTuple ( args, "si", &name, &unit ) )
		return NULL;
		
	self -> p -> setTexture ( name, unit );

	return Py_BuildValue ( "i", true );
}

static PyObject * getattr ( GlslProgramObject * self, char * name )
{
    PyObject * res = Py_FindMethod ( GlslProgram_methods, (PyObject*) self, name );

    if ( res )
        return res;

    PyErr_Clear();
	
    return NULL;
}

static PyObject * repr ( GlslProgramObject * self )
{
	char buf [128];
	
	sprintf ( buf, "GlslProgram" );
	
	return PyString_FromString ( buf );
}

extern	"C" void	DLL_EXPORT	initGlslProgram ()
{
	GlslProgram_Type.ob_type   = &PyType_Type;
	GlslProgram_Type.tp_new    = GlslProgram_new;
	GlslProgram_Type.tp_flags |= Py_TPFLAGS_CHECKTYPES;
	
			// Create the module and add the functions
	PyObject * m = Py_InitModule ( "GlslProgram", GlslProgram_Module_methods );
	
	Py_INCREF          ( &GlslProgram_Type );
    PyModule_AddObject ( m, "GlslProgram", (PyObject *)&GlslProgram_Type );
}
	
Data * getFile ( const char * fileName )
{
	return new Data ( fileName );
}

	

